#version 150

#define MAX_NUM_LIGHTS 3

// -- TYPES ----------------------------------------------------------------------------------------------------------//

struct Light {
	vec3 position;	// in world coord

	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	
	float constantAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;
};

// -- UNIFORMS -------------------------------------------------------------------------------------------------------//

// geometric transformations
uniform mat4 modelViewMatrix;
uniform mat4 modelViewProjectionMatrix;
uniform mat3 normalMatrix;
uniform vec3 camPos;

// lighting
uniform Light[MAX_NUM_LIGHTS] lights;
uniform int                   usedLights;

// -- ATTRIBUTES -----------------------------------------------------------------------------------------------------//

in vec3 vertex;
in vec3 normal;
in vec2 texCoord;

out vec3  fragmentLightDirections[MAX_NUM_LIGHTS];
out float fragmentAttenuations[MAX_NUM_LIGHTS];
out vec3  fragmentViewVector;
out vec3  fragmentNormal;
out vec2  fragmentTexCoord;

void main() {
	gl_Position = modelViewProjectionMatrix * vec4( vertex, 1 );
	
	// attributes for lighting
	//vec3 viewVertex = (modelViewMatrix * vec4(vertex, 1)).xyz;
	
	fragmentViewVector	 = normalize( camPos - vertex );
	fragmentNormal		 = normalize( normalMatrix * normal );
	fragmentTexCoord	 = texCoord;

	for ( int i = 0; i < usedLights; i++ ) {
		vec3 lightDirection = lights[i].position - vertex;
		
		fragmentLightDirections[i] = normalize( lightDirection );
		
		float distance  = length( lightDirection );
		float constant  = lights[i].constantAttenuation;
		float linear    = lights[i].linearAttenuation * distance;
		float quadratic = lights[i].quadraticAttenuation * distance * distance;
		
		fragmentAttenuations[i] = 1 / (constant + linear + quadratic);
	}
}
